/**
 * DevExtreme (ui/rich_text_editor/ui.rich_text_editor.js)
 * Version: 18.2.0 (build 18267)
 * Build date: Mon Sep 24 2018
 *
 * Copyright (c) 2012 - 2018 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _renderer = require("../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _extend = require("../../core/utils/extend");
var _type = require("../../core/utils/type");
var _dom = require("../../core/utils/dom");
var _component_registrator = require("../../core/component_registrator");
var _component_registrator2 = _interopRequireDefault(_component_registrator);
var _empty_template = require("../widget/empty_template");
var _empty_template2 = _interopRequireDefault(_empty_template);
var _editor = require("../editor/editor");
var _editor2 = _interopRequireDefault(_editor);
var _ui = require("../widget/ui.errors");
var _ui2 = _interopRequireDefault(_ui);
var _quill_registrator = require("./quill_registrator");
var _quill_registrator2 = _interopRequireDefault(_quill_registrator);
require("./converters/delta");
var _converterController = require("./converterController");
var _converterController2 = _interopRequireDefault(_converterController);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}
var RICH_TEXT_EDITOR_CLASS = "dx-richtexteditor";
var ANONYMOUS_TEMPLATE_NAME = "content";
var RichTextEditor = _editor2.default.inherit({
    _getDefaultOptions: function() {
        return (0, _extend.extend)(this.callBase(), {
            valueType: "HTML",
            placeholder: "",
            toolbar: null,
            mention: null,
            allowImageResizing: false,
            resizing: null
        })
    },
    _init: function() {
        this.callBase();
        this._quillRegistrator = new _quill_registrator2.default;
        this._prepareConverters()
    },
    _prepareConverters: function() {
        if (!this._deltaConverter) {
            var DeltaConverter = _converterController2.default.getConverter("delta");
            if (DeltaConverter) {
                this._deltaConverter = new DeltaConverter
            } else {
                throw _ui2.default.Error("E1050", "delta")
            }
        }
        if ("Markdown" === this.option("valueType") && !this._markdownConverter) {
            var MarkdownConverter = _converterController2.default.getConverter("markdown");
            if (MarkdownConverter) {
                this._markdownConverter = new MarkdownConverter
            } else {
                throw _ui2.default.Error("E1050", "markdown")
            }
        }
    },
    _getAnonymousTemplateName: function() {
        return ANONYMOUS_TEMPLATE_NAME
    },
    _initTemplates: function() {
        this.callBase();
        this._defaultTemplates.content = new _empty_template2.default(this)
    },
    _initMarkup: function() {
        var template = this._getTemplate("content");
        this._$htmlContainer = (0, _renderer2.default)("<div>");
        this.$element().addClass(RICH_TEXT_EDITOR_CLASS).wrapInner(this._$htmlContainer);
        template && template.render({
            container: (0, _dom.getPublicElement)(this._$htmlContainer),
            noModel: true
        });
        this.callBase();
        this._updateContainerMarkup()
    },
    _updateContainerMarkup: function() {
        var markup = this.option("value");
        if (this._isMarkdownValue()) {
            markup = this._markdownConverter.toHtml(markup)
        }
        if (markup) {
            this._$htmlContainer.html(markup)
        }
    },
    _render: function() {
        this._renderRichTextEditor();
        this.callBase()
    },
    _renderRichTextEditor: function() {
        var modulesConfig = this._getModulesConfig();
        this._quillInstance = this._quillRegistrator.createEditor(this._$htmlContainer[0], {
            placeholder: this.option("placeholder"),
            readOnly: this.option("readOnly") || this.option("disabled"),
            modules: modulesConfig,
            theme: "basic"
        });
        this._textChangeHandlerWithContext = this._textChangeHandler.bind(this);
        this._quillInstance.on("text-change", this._textChangeHandlerWithContext)
    },
    _getModulesConfig: function() {
        var modulesConfig = (0, _extend.extend)({}, {
            mention: this._getModuleConfigByOption("mention"),
            toolbar: this._getToolbarConfig(),
            dataPlaceholders: this._getModuleConfigByOption("dataPlaceholders")
        });
        if (this.option("allowImageResizing")) {
            modulesConfig.resizing = {
                editorInstance: this
            }
        }
        return modulesConfig
    },
    _getToolbarConfig: function() {
        var toolbarConfig = this.option("toolbar");
        var resultConfig = void 0;
        if (Array.isArray(toolbarConfig)) {
            resultConfig = {
                editorInstance: this,
                items: toolbarConfig
            }
        } else {
            resultConfig = this._getModuleConfigByOption("toolbar")
        }
        return resultConfig
    },
    _getModuleConfigByOption: function(userOptionName) {
        var userConfig = this.option(userOptionName);
        if (!(0, _type.isDefined)(userConfig)) {
            return
        }
        return (0, _extend.extend)({
            editorInstance: this
        }, userConfig)
    },
    _textChangeHandler: function(newDelta, oldDelta, source) {
        var delta = this._quillInstance.getContents(),
            htmlMarkup = this._deltaConverter.toHtml(delta.ops);
        this._isEditorUpdating = true;
        var value = this._isMarkdownValue() ? this._updateValueByType("Markdown", htmlMarkup) : htmlMarkup;
        this.option("value", value)
    },
    _updateValueByType: function(valueType, value) {
        var converter = this._markdownConverter;
        if (!(0, _type.isDefined)(converter)) {
            return
        }
        var currentValue = value || this.option("value");
        return "Markdown" === valueType ? converter.toMarkdown(currentValue) : converter.toHtml(currentValue)
    },
    _isMarkdownValue: function() {
        return "Markdown" === this.option("valueType")
    },
    _resetEnabledState: function() {
        if (this._quillInstance) {
            var isEnabled = !(this.option("readOnly") || this.option("disabled"));
            this._quillInstance.enable(isEnabled)
        }
    },
    _optionChanged: function(args) {
        switch (args.name) {
            case "value":
                if (!this._quillInstance) {
                    this._$htmlContainer.html(args.value);
                    return
                }
                if (this._isEditorUpdating) {
                    delete this._isEditorUpdating
                } else {
                    var updatedValue = this._isMarkdownValue() ? this._updateValueByType("HTML", args.value) : args.value;
                    var newDelta = this._quillInstance.clipboard.convert(updatedValue);
                    this._quillInstance.setContents(newDelta)
                }
                this.callBase(args);
                break;
            case "placeholder":
            case "mention":
            case "dataPlaceholders":
            case "allowImageResizing":
            case "resizing":
            case "toolbar":
                this._invalidate();
                break;
            case "valueType":
                this._prepareConverters();
                this.option("value", this._updateValueByType(args.value));
                break;
            case "readOnly":
            case "disabled":
                this.callBase(args);
                this._resetEnabledState();
                break;
            default:
                this.callBase(args)
        }
    },
    _clean: function() {
        if (this._quillInstance) {
            this._quillInstance.off("text-change", this._textChangeHandlerWithContext)
        }
        this.callBase()
    },
    registerModules: function(modules) {
        _quill_registrator2.default.registerModules(modules)
    }
});
(0, _component_registrator2.default)("dxRichTextEditor", RichTextEditor);
module.exports = RichTextEditor;
